@@ -1,49 +1,55 @@
-from operator import mul
-from functools import reduce
+"""Handles genomes (individuals in the population)."""
+from __future__ import division, print_function
 
-from neat.config import ConfigParameter, write_pretty_params
-from neat.genes import DefaultConnectionGene, DefaultNodeGene
-from neat.six_util import iteritems, iterkeys
-
-from neat.activations import ActivationFunctionSet
-from neat.graphs import creates_cycle
 
+from itertools import count
 from random import choice, random, shuffle
 
+import sys
 
-def product(x):
-    return reduce(mul, x, 1.0)
+from neat.activations import ActivationFunctionSet
+from neat.aggregations import AggregationFunctionSet
+from neat.config import ConfigParameter, write_pretty_params
+from neat.genes import DefaultConnectionGene, DefaultNodeGene
+from neat.graphs import creates_cycle
+from neat.six_util import iteritems, iterkeys
 
 
 class DefaultGenomeConfig(object):
-    __params = [ConfigParameter('num_inputs', int),
-                ConfigParameter('num_outputs', int),
-                ConfigParameter('num_hidden', int),
-                ConfigParameter('feed_forward', bool),
-                ConfigParameter('compatibility_disjoint_coefficient', float),
-                ConfigParameter('compatibility_weight_coefficient', float),
-                ConfigParameter('conn_add_prob', float),
-                ConfigParameter('conn_delete_prob', float),
-                ConfigParameter('node_add_prob', float),
-                ConfigParameter('node_delete_prob', float)]
-
-    allowed_connectivity = ['unconnected', 'fs_neat', 'full', 'partial']
-    aggregation_function_defs = {'sum': sum, 'max': max, 'min': min, 'product': product}
+    """Sets up and holds configuration information for the DefaultGenome class."""
+    allowed_connectivity = ['unconnected', 'fs_neat_nohidden', 'fs_neat', 'fs_neat_hidden',
+                            'full_nodirect', 'full', 'full_direct',
+                            'partial_nodirect', 'partial', 'partial_direct']
 
     def __init__(self, params):
         # Create full set of available activation functions.
         self.activation_defs = ActivationFunctionSet()
-        self.activation_options = params.get('activation_options', 'sigmoid').strip().split()
-        self.aggregation_options = params.get('aggregation_options', 'sum').strip().split()
+        # ditto for aggregation functions - name difference for backward compatibility
+        self.aggregation_function_defs = AggregationFunctionSet()
+        self.aggregation_defs = self.aggregation_function_defs
+
+        self._params = [ConfigParameter('num_inputs', int),
+                        ConfigParameter('num_outputs', int),
+                        ConfigParameter('num_hidden', int),
+                        ConfigParameter('feed_forward', bool),
+                        ConfigParameter('compatibility_disjoint_coefficient', float),
+                        ConfigParameter('compatibility_weight_coefficient', float),
+                        ConfigParameter('conn_add_prob', float),
+                        ConfigParameter('conn_delete_prob', float),
+                        ConfigParameter('node_add_prob', float),
+                        ConfigParameter('node_delete_prob', float),
+                        ConfigParameter('single_structural_mutation', bool, 'false'),
+                        ConfigParameter('structural_mutation_surer', str, 'default'),
+                        ConfigParameter('initial_connection', str, 'unconnected')]
 
         # Gather configuration data from the gene classes.
         self.node_gene_type = params['node_gene_type']
-        self.__params += self.node_gene_type.get_config_params()
+        self._params += self.node_gene_type.get_config_params()
         self.connection_gene_type = params['connection_gene_type']
-        self.__params += self.connection_gene_type.get_config_params()
+        self._params += self.connection_gene_type.get_config_params()
 
         # Use the configuration data to interpret the supplied parameters.
-        for p in self.__params:
+        for p in self._params:
             setattr(self, p.name, p.interpret(params))
 
         # By convention, input pins have negative keys, and the output
@@ -54,23 +60,43 @@ class DefaultGenomeConfig(object):
         self.connection_fraction = None
 
         # Verify that initial connection type is valid.
-        self.initial_connection = params.get('initial_connection', 'unconnected')
+        # pylint: disable=access-member-before-definition
         if 'partial' in self.initial_connection:
             c, p = self.initial_connection.split()
             self.initial_connection = c
             self.connection_fraction = float(p)
             if not (0 <= self.connection_fraction <= 1):
-                raise Exception("'partial' connection value must be between 0.0 and 1.0, inclusive.")
+                raise RuntimeError(
+                    "'partial' connection value must be between 0.0 and 1.0, inclusive.")
 
         assert self.initial_connection in self.allowed_connectivity
 
+        # Verify structural_mutation_surer is valid.
+        # pylint: disable=access-member-before-definition
+        if self.structural_mutation_surer.lower() in ['1','yes','true','on']:
+            self.structural_mutation_surer = 'true'
+        elif self.structural_mutation_surer.lower() in ['0','no','false','off']:
+            self.structural_mutation_surer = 'false'
+        elif self.structural_mutation_surer.lower() == 'default':
+            self.structural_mutation_surer = 'default'
+        else:
+            error_string = "Invalid structural_mutation_surer {!r}".format(
+                self.structural_mutation_surer)
+            raise RuntimeError(error_string)
+
+        self.node_indexer = None
+
     def add_activation(self, name, func):
         self.activation_defs.add(name, func)
 
+    def add_aggregation(self, name, func):
+        self.aggregation_function_defs.add(name, func)
+
     def save(self, f):
         if 'partial' in self.initial_connection:
             if not (0 <= self.connection_fraction <= 1):
-                raise Exception("'partial' connection value must be between 0.0 and 1.0, inclusive.")
+                raise RuntimeError(
+                    "'partial' connection value must be between 0.0 and 1.0, inclusive.")
             f.write('initial_connection      = {0} {1}\n'.format(self.initial_connection,
                                                                  self.connection_fraction))
         else:
@@ -78,8 +104,30 @@ class DefaultGenomeConfig(object):
 
         assert self.initial_connection in self.allowed_connectivity
 
-        write_pretty_params(f, self, self.__params)
+        write_pretty_params(f, self, [p for p in self._params
+                                      if not 'initial_connection' in p.name])
 
+    def get_new_node_key(self, node_dict):
+        if self.node_indexer is None:
+            self.node_indexer = count(max(list(iterkeys(node_dict))) + 1)
+
+        new_id = next(self.node_indexer)
+
+        assert new_id not in node_dict
+
+        return new_id
+
+    def check_structural_mutation_surer(self):
+        if self.structural_mutation_surer == 'true':
+            return True
+        elif self.structural_mutation_surer == 'false':
+            return False
+        elif self.structural_mutation_surer == 'default':
+            return self.single_structural_mutation
+        else:
+            error_string = "Invalid structural_mutation_surer {!r}".format(
+                self.structural_mutation_surer)
+            raise RuntimeError(error_string)
 
 class DefaultGenome(object):
     """
@@ -134,21 +182,59 @@ class DefaultGenome(object):
         # Add hidden nodes if requested.
         if config.num_hidden > 0:
             for i in range(config.num_hidden):
-                node_key = self.get_new_node_key()
+                node_key = config.get_new_node_key(self.nodes)
                 assert node_key not in self.nodes
                 node = self.create_node(config, node_key)
                 self.nodes[node_key] = node
 
         # Add connections based on initial connectivity type.
-        if config.initial_connection == 'fs_neat':
-            self.connect_fs_neat(config)
-        elif config.initial_connection == 'full':
-            self.connect_full(config)
-        elif config.initial_connection == 'partial':
-            self.connect_partial(config)
+
+        if 'fs_neat' in config.initial_connection:
+            if config.initial_connection == 'fs_neat_nohidden':
+                self.connect_fs_neat_nohidden(config)
+            elif config.initial_connection == 'fs_neat_hidden':
+                self.connect_fs_neat_hidden(config)
+            else:
+                if config.num_hidden > 0:
+                    print(
+                        "Warning: initial_connection = fs_neat will not connect to hidden nodes;",
+                        "\tif this is desired, set initial_connection = fs_neat_nohidden;",
+                        "\tif not, set initial_connection = fs_neat_hidden",
+                        sep='\n', file=sys.stderr);
+                self.connect_fs_neat_nohidden(config)
+        elif 'full' in config.initial_connection:
+            if config.initial_connection == 'full_nodirect':
+                self.connect_full_nodirect(config)
+            elif config.initial_connection == 'full_direct':
+                self.connect_full_direct(config)
+            else:
+                if config.num_hidden > 0:
+                    print(
+                        "Warning: initial_connection = full with hidden nodes will not do direct input-output connections;",
+                        "\tif this is desired, set initial_connection = full_nodirect;",
+                        "\tif not, set initial_connection = full_direct",
+                        sep='\n', file=sys.stderr);
+                self.connect_full_nodirect(config)
+        elif 'partial' in config.initial_connection:
+            if config.initial_connection == 'partial_nodirect':
+                self.connect_partial_nodirect(config)
+            elif config.initial_connection == 'partial_direct':
+                self.connect_partial_direct(config)
+            else:
+                if config.num_hidden > 0:
+                    print(
+                        "Warning: initial_connection = partial with hidden nodes will not do direct input-output connections;",
+                        "\tif this is desired, set initial_connection = partial_nodirect {0};".format(
+                            config.connection_fraction),
+                        "\tif not, set initial_connection = partial_direct {0}".format(
+                            config.connection_fraction),
+                        sep='\n', file=sys.stderr);
+                self.connect_partial_nodirect(config)
 
     def configure_crossover(self, genome1, genome2, config):
         """ Configure a new genome by crossover from two parent genomes. """
+        assert isinstance(genome1.fitness, (int, float))
+        assert isinstance(genome2.fitness, (int, float))
         if genome1.fitness > genome2.fitness:
             parent1, parent2 = genome1, genome2
         else:
@@ -181,19 +267,32 @@ class DefaultGenome(object):
     def mutate(self, config):
         """ Mutates this genome. """
 
-        # TODO: Make a configuration item to choose whether or not multiple
-        # mutations can happen simultaneously.
-        if random() < config.node_add_prob:
-            self.mutate_add_node(config)
+        if config.single_structural_mutation:
+            div = max(1,(config.node_add_prob + config.node_delete_prob +
+                         config.conn_add_prob + config.conn_delete_prob))
+            r = random()
+            if r < (config.node_add_prob/div):
+                self.mutate_add_node(config)
+            elif r < ((config.node_add_prob + config.node_delete_prob)/div):
+                self.mutate_delete_node(config)
+            elif r < ((config.node_add_prob + config.node_delete_prob +
+                       config.conn_add_prob)/div):
+                self.mutate_add_connection(config)
+            elif r < ((config.node_add_prob + config.node_delete_prob +
+                       config.conn_add_prob + config.conn_delete_prob)/div):
+                self.mutate_delete_connection()
+        else:
+            if random() < config.node_add_prob:
+                self.mutate_add_node(config)
 
-        if random() < config.node_delete_prob:
-            self.mutate_delete_node(config)
+            if random() < config.node_delete_prob:
+                self.mutate_delete_node(config)
 
-        if random() < config.conn_add_prob:
-            self.mutate_add_connection(config)
+            if random() < config.conn_add_prob:
+                self.mutate_add_connection(config)
 
-        if random() < config.conn_delete_prob:
-            self.mutate_delete_connection()
+            if random() < config.conn_delete_prob:
+                self.mutate_delete_connection()
 
         # Mutate connection genes.
         for cg in self.connections.values():
@@ -203,19 +302,15 @@ class DefaultGenome(object):
         for ng in self.nodes.values():
             ng.mutate(config)
 
-    def get_new_node_key(self):
-        new_id = 0
-        while new_id in self.nodes:
-            new_id += 1
-        return new_id
-
     def mutate_add_node(self, config):
         if not self.connections:
-            return None, None
+            if config.check_structural_mutation_surer():
+                self.mutate_add_connection(config)
+            return
 
         # Choose a random connection to split
         conn_to_split = choice(list(self.connections.values()))
-        new_node_id = self.get_new_node_key()
+        new_node_id = config.get_new_node_key(self.nodes)
         ng = self.create_node(config, new_node_id)
         self.nodes[new_node_id] = ng
 
@@ -229,7 +324,11 @@ class DefaultGenome(object):
         self.add_connection(config, new_node_id, o, conn_to_split.weight, True)
 
     def add_connection(self, config, input_key, output_key, weight, enabled):
-        # TODO: Add validation of this connection addition.
+        # TODO: Add further validation of this connection addition?
+        assert isinstance(input_key, int)
+        assert isinstance(output_key, int)
+        assert output_key >= 0
+        assert isinstance(enabled, bool)
         key = (input_key, output_key)
         connection = config.connection_gene_type(key)
         connection.init_attributes(config)
@@ -238,10 +337,10 @@ class DefaultGenome(object):
         self.connections[key] = connection
 
     def mutate_add_connection(self, config):
-        '''
+        """
         Attempt to add a new connection, the only restriction being that the output
         node cannot be one of the network input pins.
-        '''
+        """
         possible_outputs = list(iterkeys(self.nodes))
         out_node = choice(possible_outputs)
 
@@ -251,8 +350,18 @@ class DefaultGenome(object):
         # Don't duplicate connections.
         key = (in_node, out_node)
         if key in self.connections:
+            # TODO: Should this be using mutation to/from rates? Hairy to configure...
+            if config.check_structural_mutation_surer():
+                self.connections[key].enabled = True
             return
 
+        # Don't allow connections between two output nodes
+        if in_node in config.output_keys and out_node in config.output_keys:
+            return
+
+        # No need to check for connections between input nodes:
+        # they cannot be the output end of a connection (see above).
+
         # For feed-forward networks, avoid creating cycles.
         if config.feed_forward and creates_cycle(list(iterkeys(self.connections)), key):
             return
@@ -262,11 +371,11 @@ class DefaultGenome(object):
 
     def mutate_delete_node(self, config):
         # Do nothing if there are no non-output nodes.
-        available_nodes = [(k, v) for k, v in iteritems(self.nodes) if k not in config.output_keys]
+        available_nodes = [k for k in iterkeys(self.nodes) if k not in config.output_keys]
         if not available_nodes:
             return -1
 
-        del_key, del_node = choice(available_nodes)
+        del_key = choice(available_nodes)
 
         connections_to_delete = set()
         for k, v in iteritems(self.connections):
@@ -308,7 +417,9 @@ class DefaultGenome(object):
                     node_distance += n1.distance(n2, config)
 
             max_nodes = max(len(self.nodes), len(other.nodes))
-            node_distance = (node_distance + config.compatibility_disjoint_coefficient * disjoint_nodes) / max_nodes
+            node_distance = (node_distance +
+                             (config.compatibility_disjoint_coefficient *
+                              disjoint_nodes)) / max_nodes
 
         # Compute connection gene differences.
         connection_distance = 0.0
@@ -327,19 +438,23 @@ class DefaultGenome(object):
                     connection_distance += c1.distance(c2, config)
 
             max_conn = max(len(self.connections), len(other.connections))
-            conn_sum = connection_distance + config.compatibility_disjoint_coefficient * disjoint_connections
-            connection_distance = conn_sum / max_conn
+            connection_distance = (connection_distance +
+                                   (config.compatibility_disjoint_coefficient *
+                                    disjoint_connections)) / max_conn
 
         distance = node_distance + connection_distance
         return distance
 
     def size(self):
-        '''Returns genome 'complexity', taken to be (number of nodes, number of enabled connections)'''
-        num_enabled_connections = sum([1 for cg in self.connections.values() if cg.enabled is True])
+        """
+        Returns genome 'complexity', taken to be
+        (number of nodes, number of enabled connections)
+        """
+        num_enabled_connections = sum([1 for cg in self.connections.values() if cg.enabled])
         return len(self.nodes), num_enabled_connections
 
     def __str__(self):
-        s = "Nodes:"
+        s = "Key: {0}\nFitness: {1}\nNodes:".format(self.key, self.fitness)
         for k, ng in iteritems(self.nodes):
             s += "\n\t{0} {1!s}".format(k, ng)
         s += "\nConnections:"
@@ -361,18 +476,35 @@ class DefaultGenome(object):
         connection.init_attributes(config)
         return connection
 
-    def connect_fs_neat(self, config):
-        """ Randomly connect one input to all hidden and output nodes (FS-NEAT). """
+    def connect_fs_neat_nohidden(self, config):
+        """
+        Randomly connect one input to all output nodes
+        (FS-NEAT without connections to hidden, if any).
+        Originally connect_fs_neat.
+        """
         input_id = choice(config.input_keys)
         for output_id in config.output_keys:
             connection = self.create_connection(config, input_id, output_id)
             self.connections[connection.key] = connection
 
-    def compute_full_connections(self, config):
+    def connect_fs_neat_hidden(self, config):
+        """
+        Randomly connect one input to all hidden and output nodes
+        (FS-NEAT with connections to hidden, if any).
+        """
+        input_id = choice(config.input_keys)
+        others = [i for i in iterkeys(self.nodes) if i not in config.input_keys]
+        for output_id in others:
+            connection = self.create_connection(config, input_id, output_id)
+            self.connections[connection.key] = connection
+
+    def compute_full_connections(self, config, direct):
         """
         Compute connections for a fully-connected feed-forward genome--each
-        input connected to all hidden nodes, each hidden node connected to all
-        output nodes.
+        input connected to all hidden nodes
+        (and output nodes if ``direct`` is set or there are no hidden nodes),
+        each hidden node connected to all output nodes.
+        (Recurrent genomes will also include node self-connections.)
         """
         hidden = [i for i in iterkeys(self.nodes) if i not in config.output_keys]
         output = [i for i in iterkeys(self.nodes) if i in config.output_keys]
@@ -384,22 +516,53 @@ class DefaultGenome(object):
             for h in hidden:
                 for output_id in output:
                     connections.append((h, output_id))
-        else:
+        if direct or (not hidden):
             for input_id in config.input_keys:
                 for output_id in output:
                     connections.append((input_id, output_id))
 
+        # For recurrent genomes, include node self-connections.
+        if not config.feed_forward:
+            for i in iterkeys(self.nodes):
+                connections.append((i, i))
+
         return connections
 
-    def connect_full(self, config):
-        """ Create a fully-connected genome. """
-        for input_id, output_id in self.compute_full_connections(config):
+
+    def connect_full_nodirect(self, config):
+        """
+        Create a fully-connected genome
+        (except without direct input-output unless no hidden nodes).
+        """
+        for input_id, output_id in self.compute_full_connections(config, False):
             connection = self.create_connection(config, input_id, output_id)
             self.connections[connection.key] = connection
 
-    def connect_partial(self, config):
+    def connect_full_direct(self, config):
+        """ Create a fully-connected genome, including direct input-output connections. """
+        for input_id, output_id in self.compute_full_connections(config, True):
+            connection = self.create_connection(config, input_id, output_id)
+            self.connections[connection.key] = connection
+
+    def connect_partial_nodirect(self, config):
+        """
+        Create a partially-connected genome,
+        with (unless no hidden nodes) no direct input-output connections."""
         assert 0 <= config.connection_fraction <= 1
-        all_connections = self.compute_full_connections(config)
+        all_connections = self.compute_full_connections(config, False)
+        shuffle(all_connections)
+        num_to_add = int(round(len(all_connections) * config.connection_fraction))
+        for input_id, output_id in all_connections[:num_to_add]:
+            connection = self.create_connection(config, input_id, output_id)
+            self.connections[connection.key] = connection
+
+    def connect_partial_direct(self, config):
+        """
+        Create a partially-connected genome,
+        including (possibly) direct input-output connections.
+        """
+        assert 0 <= config.connection_fraction <= 1
+        all_connections = self.compute_full_connections(config, True)
         shuffle(all_connections)
         num_to_add = int(round(len(all_connections) * config.connection_fraction))
         for input_id, output_id in all_connections[:num_to_add]:
