@@ -1,7 +1,7 @@
 """
 This module implements a spiking neural network.
 Neurons are based on the model described by:
-    
+
 Izhikevich, E. M.
 Simple Model of Spiking Neurons
 IEEE TRANSACTIONS ON NEURAL NETWORKS, VOL. 14, NO. 6, NOVEMBER 2003
@@ -10,10 +10,10 @@ http://www.izhikevich.org/publications/spikes.pdf
 """
 
 from neat.attributes import FloatAttribute
-from neat.genes import BaseGene, DefaultGeneConfig, DefaultConnectionGene
+from neat.genes import BaseGene, DefaultConnectionGene
 from neat.genome import DefaultGenomeConfig, DefaultGenome
 from neat.graphs import required_for_output
-from neat.six_util import itervalues, iteritems
+from neat.six_util import itervalues
 
 # a, b, c, d are the parameters of the Izhikevich model.
 # a: the time scale of the recovery variable
@@ -21,6 +21,7 @@ from neat.six_util import itervalues, iteritems
 # c: the after-spike reset value of the membrane potential
 # d: after-spike reset of the recovery variable
 # The following parameter sets produce some known spiking behaviors:
+# pylint: disable=bad-whitespace
 REGULAR_SPIKING_PARAMS        = {'a': 0.02, 'b': 0.20, 'c': -65.0, 'd': 8.00}
 INTRINSICALLY_BURSTING_PARAMS = {'a': 0.02, 'b': 0.20, 'c': -55.0, 'd': 4.00}
 CHATTERING_PARAMS             = {'a': 0.02, 'b': 0.20, 'c': -50.0, 'd': 2.00}
@@ -34,16 +35,14 @@ LOW_THRESHOLD_SPIKING_PARAMS  = {'a': 0.02, 'b': 0.25, 'c': -65.0, 'd': 2.00}
 
 
 class IZNodeGene(BaseGene):
-    __gene_attributes__ = [FloatAttribute('bias'),
+    """Contains attributes for the iznn node genes and determines genomic distances."""
+
+    _gene_attributes = [FloatAttribute('bias'),
                            FloatAttribute('a'),
                            FloatAttribute('b'),
                            FloatAttribute('c'),
                            FloatAttribute('d')]
 
-    @classmethod
-    def parse_config(cls, config, param_dict):
-        return DefaultGeneConfig(cls.__gene_attributes__, param_dict)
-
     def distance(self, other, config):
         s = abs(self.a - other.a) + abs(self.b - other.b) \
             + abs(self.c - other.c) + abs(self.d - other.d)
@@ -51,18 +50,26 @@ class IZNodeGene(BaseGene):
 
 
 class IZGenome(DefaultGenome):
-    @staticmethod
-    def parse_config(param_dict):
+    @classmethod
+    def parse_config(cls, param_dict):
         param_dict['node_gene_type'] = IZNodeGene
         param_dict['connection_gene_type'] = DefaultConnectionGene
         return DefaultGenomeConfig(param_dict)
 
 
 class IZNeuron(object):
+    """Sets up and simulates the iznn nodes (neurons)."""
     def __init__(self, bias, a, b, c, d, inputs):
         """
         a, b, c, d are the parameters of the Izhikevich model.
-        inputs: list of (input key, weight) pairs for incoming connections
+
+        :param float bias: The bias of the neuron.
+        :param float a: The time-scale of the recovery variable.
+        :param float b: The sensitivity of the recovery variable.
+        :param float c: The after-spike reset value of the membrane potential.
+        :param float d: The after-spike reset value of the recovery variable.
+        :param inputs: A list of (input key, weight) pairs for incoming connections.
+        :type inputs: list(tuple(int, float))
         """
         self.a = a
         self.b = b
@@ -119,6 +126,7 @@ class IZNeuron(object):
 
 
 class IZNN(object):
+    """Basic iznn network object."""
     def __init__(self, neurons, inputs, outputs):
         self.neurons = neurons
         self.inputs = inputs
@@ -127,16 +135,22 @@ class IZNN(object):
 
     def set_inputs(self, inputs):
         """Assign input voltages."""
-        assert len(inputs) == len(self.inputs)
+        if len(inputs) != len(self.inputs):
+            raise RuntimeError(
+                "Number of inputs {0:d} does not match number of input nodes {1:d}".format(
+                    len(inputs),len(self.inputs)))
         for i, v in zip(self.inputs, inputs):
             self.input_values[i] = v
 
     def reset(self):
         """Reset all neurons to their default state."""
-        for i, n in self.neurons.items():
+        for n in itervalues(self.neurons):
             n.reset()
 
     def get_time_step_msec(self):
+        # pylint: disable=no-self-use
+        # TODO: Investigate performance or numerical stability issues that may
+        # result from using this hard-coded time step.
         return 0.05
 
     def advance(self, dt_msec):
