@@ -1,26 +1,30 @@
+"""
+Runs evaluation functions in parallel subprocesses
+in order to evaluate multiple genomes at once.
+"""
 from multiprocessing import Pool
 
-
 class ParallelEvaluator(object):
     def __init__(self, num_workers, eval_function, timeout=None):
-        '''
-        eval_function should take one argument (a genome object) and return
+        """
+        eval_function should take one argument, a tuple of
+        (genome object, config object), and return
         a single float (the genome's fitness).
-        '''
+        """
         self.num_workers = num_workers
         self.eval_function = eval_function
         self.timeout = timeout
         self.pool = Pool(num_workers)
 
     def __del__(self):
-        self.pool.close()
+        self.pool.close() # should this be terminate?
         self.pool.join()
 
     def evaluate(self, genomes, config):
         jobs = []
-        for genome_id, genome in genomes:
+        for ignored_genome_id, genome in genomes:
             jobs.append(self.pool.apply_async(self.eval_function, (genome, config)))
 
         # assign the fitness back to each genome
-        for job, (genome_id, genome) in zip(jobs, genomes):
+        for job, (ignored_genome_id, genome) in zip(jobs, genomes):
             genome.fitness = job.get(timeout=self.timeout)
