@@ -1,13 +1,23 @@
-from __future__ import print_function
+"""
+Makes possible reporter classes,
+which are triggered on particular events and may provide information to the user,
+may do something else such as checkpointing, or may do both.
+"""
+from __future__ import division, print_function
 
 import time
 
 from neat.math_util import mean, stdev
-from neat.six_util import itervalues
+from neat.six_util import itervalues, iterkeys
 
 # TODO: Add a curses-based reporter.
 
+
 class ReporterSet(object):
+    """
+    Keeps track of the set of reporters
+    and gives methods to dispatch them at appropriate points.
+    """
     def __init__(self):
         self.reporters = []
 
@@ -21,14 +31,18 @@ class ReporterSet(object):
         for r in self.reporters:
             r.start_generation(gen)
 
-    def end_generation(self, config, population, species):
+    def end_generation(self, config, population, species_set):
         for r in self.reporters:
-            r.end_generation(config, population, species)
+            r.end_generation(config, population, species_set)
 
     def post_evaluate(self, config, population, species, best_genome):
         for r in self.reporters:
             r.post_evaluate(config, population, species, best_genome)
 
+    def post_reproduction(self, config, population, species):
+        for r in self.reporters:
+            r.post_reproduction(config, population, species)
+
     def complete_extinction(self):
         for r in self.reporters:
             r.complete_extinction()
@@ -51,12 +65,15 @@ class BaseReporter(object):
     def start_generation(self, generation):
         pass
 
-    def end_generation(self, config, population, species):
+    def end_generation(self, config, population, species_set):
         pass
 
     def post_evaluate(self, config, population, species, best_genome):
         pass
 
+    def post_reproduction(self, config, population, species):
+        pass
+
     def complete_extinction(self):
         pass
 
@@ -71,7 +88,9 @@ class BaseReporter(object):
 
 
 class StdOutReporter(BaseReporter):
-    def __init__(self):
+    """Uses `print` to output information about the run; an example reporter class."""
+    def __init__(self, show_species_detail):
+        self.show_species_detail = show_species_detail
         self.generation = None
         self.generation_start_time = None
         self.generation_times = []
@@ -82,7 +101,27 @@ class StdOutReporter(BaseReporter):
         print('\n ****** Running generation {0} ****** \n'.format(generation))
         self.generation_start_time = time.time()
 
-    def end_generation(self, config, population, species):
+    def end_generation(self, config, population, species_set):
+        ng = len(population)
+        ns = len(species_set.species)
+        if self.show_species_detail:
+            print('Population of {0:d} members in {1:d} species:'.format(ng, ns))
+            sids = list(iterkeys(species_set.species))
+            sids.sort()
+            print("   ID   age  size  fitness  adj fit  stag")
+            print("  ====  ===  ====  =======  =======  ====")
+            for sid in sids:
+                s = species_set.species[sid]
+                a = self.generation - s.created
+                n = len(s.members)
+                f = "--" if s.fitness is None else "{:.1f}".format(s.fitness)
+                af = "--" if s.adjusted_fitness is None else "{:.3f}".format(s.adjusted_fitness)
+                st = self.generation - s.last_improved
+                print(
+                    "  {: >4}  {: >3}  {: >4}  {: >7}  {: >7}  {: >4}".format(sid, a, n, f, af, st))
+        else:
+            print('Population of {0:d} members in {1:d} species'.format(ng, ns))
+
         elapsed = time.time() - self.generation_start_time
         self.generation_times.append(elapsed)
         self.generation_times = self.generation_times[-10:]
@@ -94,17 +133,17 @@ class StdOutReporter(BaseReporter):
             print("Generation time: {0:.3f} sec".format(elapsed))
 
     def post_evaluate(self, config, population, species, best_genome):
+        # pylint: disable=no-self-use
         fitnesses = [c.fitness for c in itervalues(population)]
         fit_mean = mean(fitnesses)
         fit_std = stdev(fitnesses)
         best_species_id = species.get_species_id(best_genome.key)
         print('Population\'s average fitness: {0:3.5f} stdev: {1:3.5f}'.format(fit_mean, fit_std))
-        print('Best fitness: {0:3.5f} - size: {1!r} - species {2} - id {3}'.format(best_genome.fitness, best_genome.size(),
-                                                                                   best_species_id, best_genome.key))
-        print('Species length: {0:d} totaling {1:d} individuals'.format(len(species.species), len(population)))
-        #print('Species ID       : {0!s}'.format([s.ID for s in species]))
-        #print('Species size     : {0!s}'.format([len(s.members) for s in species]))
-        #print('Species age      : {0}'.format([s.age for s in species]))
+        print(
+            'Best fitness: {0:3.5f} - size: {1!r} - species {2} - id {3}'.format(best_genome.fitness,
+                                                                                 best_genome.size(),
+                                                                                 best_species_id,
+                                                                                 best_genome.key))
 
     def complete_extinction(self):
         self.num_extinctions += 1
@@ -115,7 +154,8 @@ class StdOutReporter(BaseReporter):
             self.generation, best.size()))
 
     def species_stagnant(self, sid, species):
-        print("\nSpecies {0} with {1} members is stagnated: removing it".format(sid, len(species.members)))
+        if self.show_species_detail:
+            print("\nSpecies {0} with {1} members is stagnated: removing it".format(sid, len(species.members)))
 
     def info(self, msg):
         print(msg)
